-- Zoom speed (factor to zoom in/out per second)
local zoomFactorPerSec = 2
-- Precalculate this value for later
local lambdaZoom = math.log(zoomFactorPerSec)

-- Image
local img

-- Camera parameters
local ox, oy
local zoom

-- Convert screen coordinates to image coordinates
local function screenToImage(x, y)
  return (x - ox) / zoom, (y - oy) / zoom
end

function love.load()
  img = love.graphics.newImage('image.jpg')
  ox = (love.graphics.getWidth() - img:getWidth()) / 2
  oy = (love.graphics.getHeight() - img:getHeight()) / 2
  zoom = 1
end

function love.update(dt)
  local lButton, rButton = love.mouse.isDown(1), love.mouse.isDown(2)
  if lButton or rButton then
    local mx, my = screenToImage(love.mouse.getPosition())

    -- Offset by previous zoom
    ox = ox + mx * zoom
    oy = oy + my * zoom

    local zoomFactor = math.exp(lambdaZoom * dt)
    if lButton then
      zoom = zoom * zoomFactor
    end
    if rButton then
      zoom = zoom / zoomFactor
    end

    -- Offset by new zoom
    ox = ox - mx * zoom
    oy = oy - my * zoom
  end
end

function love.draw()
  love.graphics.translate(ox, oy)
  love.graphics.scale(zoom)
  love.graphics.draw(img)

  love.graphics.origin()
  love.graphics.print(string.format("%g %g", screenToImage(love.mouse.getX(), love.mouse.getY())))
end

function love.keypressed(k) return k == "escape" and love.event.quit() end
